#include <iostream>
#include <algorithm>
#include <utility>
#include <vector>
#include <array>
#include <random>
#include <cmath>

#include "solution.h"

#define SET_VAL 0
#define ADD_VAL 1

const unsigned int MEMORY_C = 5;

void dfs(
    int u,
    const std::vector<std::vector<unsigned int>> &adj,
    std::vector<bool> &used
) {
    used[u] = true;
    for(auto v: adj[u]) {
        if(!used[v]) {
            dfs(v, adj, used);
        }
    }
}

int main() {
    unsigned int n, m, seed;
    std::cin >> n >> m >> seed;
    std::mt19937 generator(seed);

    std::vector<std::vector<unsigned int>> adj;
    adj.assign(n + 1, std::vector<unsigned int>());
    for(unsigned int i = 0; i < m; i++) {
        unsigned int u, v;
        std::cin >> u >> v;
        adj[u].push_back(v);
        adj[v].push_back(u);
    }

    unsigned int *memory = new unsigned int[MEMORY_C * n + 3];
    unsigned int *next_memory = new unsigned int[MEMORY_C * n + 3];
    std::fill(memory, memory + MEMORY_C * n + 3, 0);
    memory[1] = n;
    memory[2] = m;

    unsigned int T = 0;
    while(!memory[0] && T < 200000) {
        T++;
        std::vector<unsigned int> robots;
        robots.resize(n);
        std::iota(robots.begin(), robots.end(), 1);

        // We don't have assumptions on the execution order of the robots
        std::shuffle(robots.begin(), robots.end(), generator);

        std::copy(memory, memory + MEMORY_C * n + 3, next_memory);
        for(unsigned int u: robots) {
            std::array<MemoryOperation, MEMORY_C> writes = work(u,
                    memory,
                    MEMORY_C * n + 3,
                    adj[u].data(),
                    adj[u].size());

            for(auto memory_operation: writes) {
                bool type = memory_operation.type;
                unsigned int addr = memory_operation.addr;
                unsigned int val = memory_operation.val;

                if(addr >= MEMORY_C * n + 3) {
                    std::cerr << "Invalid memory address " << addr << std::endl;
                    std::cout << -1 << std::endl;
                    return 0;
                }

                if(type == SET_VAL) {
                    next_memory[addr] = val;
                } else {
                    next_memory[addr] += val;
                }
            }
        }
        std::copy(next_memory, next_memory + MEMORY_C * n + 3, memory);
    }


    if(T == 200000) {
        std::cerr << "Too many steps" << std::endl;
        std::cout << -2 << std::endl;
        return 0;
    }

    if(memory[0] != 1 && memory[0] != 2) {
        std::cerr << "Invalid return value " << memory[0] << std::endl;
        std::cout << -3 << std::endl;
        return 0;
    }

    std::vector<bool> used(n + 1, false);
    dfs(1, adj, used);

    bool connected = true;
    for(unsigned int i = 1; i <= n; i++) {
        if(!used[i]) {
            connected = false;
        }
    }

    std::cout << connected << " " << memory[3] << std::endl;
    if(connected != 2 - memory[0]) {
        std::cerr << "Wrong answer" << std::endl;
        std::cout << 0 << std::endl;
    } else {
        std::cerr << "Success" << std::endl;
        std::cout << 1 << std::endl;
        std::cout << T << std::endl;
    }

    return 0;
}

// You can ignore this part. It is used ensure that you don't change your siganture.
// We run your solution artificially to ensure that it is constexpr:

unsigned int testConstexprMain() {
    std::array<std::array<unsigned int, 2>, 100> adj{};
    for (unsigned int i = 1; i < 100; ++i) {
        adj[i][0] = i - 1;
        adj[i][1] = i + 1;
        if(adj[i][0] == 0) {
            adj[i][0] = 99;
        }

        if(adj[i][1] == 100) {
            adj[i][1] = 1;
        }
    }

    unsigned int sum = 0;
    std::array<unsigned int, MEMORY_C * 99 + 3> memory{};
    for(unsigned int i = 0; i < 100; i++) {
        std::array<MemoryOperation, MEMORY_C> ops = work(i, memory.data(), MEMORY_C * 99 + 3, adj[i].data(), 2);
        for(unsigned int j = 0; j < MEMORY_C; j++) {
            sum += ops[j].type;
            sum += ops[j].addr;
            sum += ops[j].val;
        }
    }

    return sum;
}

unsigned int testConstexpr = testConstexprMain();


/*
5 5 1
1 2
2 3
3 4
1 4
2 4
*/
